#include <stdio.h>
#include <stdlib.h>
#include <gtk/gtk.h>

#if (GTK_MAJOR_VERSION == 1)
 #define gtk_window_set_resizable(w, b)		gtk_window_set_policy(w, FALSE, b, !b)
 #define g_signal_connect			gtk_signal_connect
 #define G_OBJECT				GTK_OBJECT
 #define G_CALLBACK				GTK_SIGNAL_FUNC
#endif

#include "caotica2.xpm"
#include "bpmcalc.xpm"
#include "error.xpm"

#define SPACING		10
#define SPACING_2	 5
#define BUFSIZE		32

static GtkWidget	*_window;
static GtkWidget	*_period_entry;
static GtkWidget	*_beats_entry;
static GtkWidget	*_tempo_entry;
static GtkWidget	*_nth_check;
static GtkWidget	*_round_check;
static GtkWidget	*_nth2_entry;
static GtkWidget	*_nth4_entry;
static GtkWidget	*_nth8_entry;
static GtkWidget	*_nth16_entry;
static GtkWidget	*_nth32_entry;
static GtkWidget	*_nth64_entry;
static GtkWidget	*_nth128_entry;
static GtkWidget	*_nth256_entry;

#if (GTK_MAJOR_VERSION == 1)
static GtkAccelGroup	*_accel_group;

static GtkWidget *gtk_button_new_with_mnemonic(const gchar *label)
{
	GtkWidget	*button;
	guint		keyval;

	button = gtk_button_new_with_label(label);
	keyval = gtk_label_parse_uline(GTK_LABEL(GTK_BIN(button)->child), label);
	gtk_accel_group_add(_accel_group, keyval, GDK_MOD1_MASK, 0, GTK_OBJECT(button), "clicked");

	return(button);
}

static GtkWidget *gtk_check_button_new_with_mnemonic(const gchar *label)
{
	GtkWidget	*button;
	guint		keyval;

	button = gtk_check_button_new_with_label(label);
	keyval = gtk_label_parse_uline(GTK_LABEL(GTK_BIN(button)->child), label);
	gtk_accel_group_add(_accel_group, keyval, GDK_MOD1_MASK, 0, GTK_OBJECT(button), "clicked");

	return(button);
}
#endif

static GtkWidget *build_pixmap(GtkWidget *window, gchar **xpm, gboolean set_icon)
{
	GtkStyle	*style;
	GdkBitmap	*mask;
	GdkPixmap	*pixels;
	GtkWidget	*pixmap;

	style = gtk_widget_get_style(window);
	pixels = gdk_pixmap_create_from_xpm_d(window->window, &mask, &style->bg[GTK_STATE_NORMAL], xpm);

	pixmap = gtk_pixmap_new(pixels, mask);

	if (set_icon)
		gdk_window_set_icon(window->window, NULL, pixels, mask);

	gdk_pixmap_unref(pixels);

	return(pixmap);
}

static void close_error(GtkWidget *widget, gpointer window)
{
	gtk_widget_destroy(window);
	return;
}

static void display_error(const char *error)
{
	GtkWidget	*window;
	GtkWidget	*vbox;
	GtkWidget	*table;
	GtkWidget	*pixmap;
	GtkWidget	*label;
	GtkWidget	*button;

	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_modal(GTK_WINDOW(window), TRUE);
	gtk_window_set_resizable(GTK_WINDOW(window), FALSE);
	gtk_window_set_title(GTK_WINDOW(window), "BPMcalc");
	gtk_container_set_border_width(GTK_CONTAINER(window), SPACING);
	gtk_widget_realize(window);

	vbox = gtk_vbox_new(FALSE, 16);
	gtk_container_add(GTK_CONTAINER(window), vbox);
	gtk_widget_show(vbox);

	table = gtk_table_new(1, 2, FALSE);
	gtk_table_set_col_spacings(GTK_TABLE(table), 16);
	gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, FALSE, 0);
	gtk_widget_show(table);

	pixmap = build_pixmap(window, (gchar **) _error_xpm, FALSE);
	gtk_table_attach_defaults(GTK_TABLE(table), pixmap, 0, 1, 0, 1);
	gtk_widget_show(pixmap);

	label = gtk_label_new(error);
	gtk_table_attach_defaults(GTK_TABLE(table), label, 1, 2, 0, 1);
	gtk_widget_show(label);

	table = gtk_table_new(1, 3, FALSE);
	gtk_table_set_col_spacings(GTK_TABLE(table), 16);
	gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, FALSE, 0);
	gtk_widget_show(table);

	button = gtk_button_new_with_label("OK");
	gtk_table_attach_defaults(GTK_TABLE(table), button, 1, 2, 0, 1);
	g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(close_error), window);
	gtk_widget_show(button);

	gdk_window_set_transient_for(window->window, _window->window);
	gtk_widget_show(window);
	return;
}

static GtkWidget *build_nth(GtkWidget *table, int row, int nth, double period, gboolean rounded)
{
	char		buf[BUFSIZE];
	GtkWidget	*label;
	GtkWidget	*entry;

	snprintf(buf, sizeof(buf), "1/%d", nth);

	label = gtk_label_new(buf);
	gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, row, row+1);
	gtk_widget_show(label);

	if (rounded)
		snprintf(buf, sizeof(buf), "%.3f", period / nth);
	else
		snprintf(buf, sizeof(buf), "%f", period / nth);

	entry = gtk_entry_new();
	gtk_entry_set_text(GTK_ENTRY(entry), buf);
	gtk_editable_set_editable(GTK_EDITABLE(entry), FALSE);
	if (!row) gtk_editable_select_region(GTK_EDITABLE(entry), 0, -1);
	gtk_table_attach_defaults(GTK_TABLE(table), entry, 1, 2, row, row+1);
	gtk_widget_show(entry);

	label = gtk_label_new("s");
	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(table), label, 2, 3, row, row+1);
	gtk_widget_show(label);

	return(entry);
}

static void display_nth(double period, gboolean rounded)
{
	GtkWidget	*window;
	GtkWidget	*table;

	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_modal(GTK_WINDOW(window), TRUE);
	gtk_window_set_resizable(GTK_WINDOW(window), FALSE);
	gtk_window_set_title(GTK_WINDOW(window), "caotica2 BPM Calculator NTH");
	gtk_container_set_border_width(GTK_CONTAINER(window), SPACING);
	gtk_widget_realize(window);

	table = gtk_table_new(8, 3, FALSE);
	gtk_table_set_row_spacings(GTK_TABLE(table), SPACING);
	gtk_table_set_col_spacing(GTK_TABLE(table), 0, SPACING);
	gtk_table_set_col_spacing(GTK_TABLE(table), 1, SPACING_2);
	gtk_container_add(GTK_CONTAINER(window), table);
	gtk_widget_show(table);

	_nth2_entry   = build_nth(table, 0,   2, period, rounded);
	_nth4_entry   = build_nth(table, 1,   4, period, rounded);
	_nth8_entry   = build_nth(table, 2,   8, period, rounded);
	_nth16_entry  = build_nth(table, 3,  16, period, rounded);
	_nth32_entry  = build_nth(table, 4,  32, period, rounded);
	_nth64_entry  = build_nth(table, 5,  64, period, rounded);
	_nth128_entry = build_nth(table, 6, 128, period, rounded);
	_nth256_entry = build_nth(table, 7, 256, period, rounded);

	gdk_window_set_transient_for(window->window, _window->window);
	gtk_widget_show(window);
	return;
}

static void compute_tempo(GtkButton *widget, gpointer data)
{
	gchar	*value;
	char	*end;
	double	period;
	double	beats;

	value = gtk_editable_get_chars(GTK_EDITABLE(_period_entry), 0, -1);
	period = strtod(value, &end);
	g_free(value);

	value = gtk_editable_get_chars(GTK_EDITABLE(_beats_entry), 0, -1);
	beats = strtod(value, &end);
	g_free(value);

	if (period && beats) {
		double		tempo;
		gboolean	rounded;
		char		buf[BUFSIZE];

		tempo = (60 / period) * beats;

		rounded = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_round_check));

		if (rounded)
			snprintf(buf, sizeof(buf), "%.3f", tempo);
		else
			snprintf(buf, sizeof(buf), "%f", tempo);

		gtk_entry_set_text(GTK_ENTRY(_tempo_entry), buf);

		if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_nth_check)))
			display_nth(period, rounded);
	}
	else
		display_error("invalid data");

	return;
}

static void compute_period(GtkButton *widget, gpointer data)
{
	gchar	*value;
	char	*end;
	double	tempo;
	double	beats;

	value = gtk_editable_get_chars(GTK_EDITABLE(_tempo_entry), 0, -1);
	tempo = strtod(value, &end);
	g_free(value);

	value = gtk_editable_get_chars(GTK_EDITABLE(_beats_entry), 0, -1);
	beats = strtod(value, &end);
	g_free(value);

	if (tempo && beats) {
		double		period;
		gboolean	rounded;
		char		buf[BUFSIZE];

		period = (60 / tempo) * beats;

		rounded = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_round_check));

		if (rounded)
			snprintf(buf, sizeof(buf), "%.3f", period);
		else
			snprintf(buf, sizeof(buf), "%f", period);

		gtk_entry_set_text(GTK_ENTRY(_period_entry), buf);

		if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_nth_check)))
			display_nth(period, rounded);
	}
	else
		display_error("invalid data");

	return;
}

static void visit_site(GtkWidget *widget, gpointer data)
{
	system("dillo http://caotica2.free.fr/ > /dev/null 2>&1 &");
	return;
}

static void close_about(GtkWidget *widget, gpointer window)
{
	gtk_widget_destroy(window);
	return;
}

static void display_about(GtkButton *widget, gpointer data)
{
	GtkWidget	*window;
	GtkWidget	*hbox;
	GtkWidget	*pixmap;
	GtkWidget	*table;
	GtkWidget	*label;
	GtkWidget	*button;

	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_modal(GTK_WINDOW(window), TRUE);
	gtk_window_set_resizable(GTK_WINDOW(window), FALSE);
	gtk_window_set_title(GTK_WINDOW(window), "About BPMcalc");
	gtk_container_set_border_width(GTK_CONTAINER(window), SPACING);
	gtk_widget_realize(window);

	hbox = gtk_hbox_new(FALSE, 16);
	gtk_container_add(GTK_CONTAINER(window), hbox);
	gtk_widget_show(hbox);

	pixmap = build_pixmap(window, (gchar **) _bpmcalc_xpm, FALSE);
	gtk_misc_set_alignment(GTK_MISC(pixmap), 0.5, 0.10);
	gtk_box_pack_start(GTK_BOX(hbox), pixmap, FALSE, FALSE, 0);
	gtk_widget_show(pixmap);

	table = gtk_table_new(4, 2, FALSE);
	gtk_table_set_row_spacing(GTK_TABLE(table), 1, 16);
	gtk_table_set_col_spacings(GTK_TABLE(table), SPACING);
	gtk_box_pack_start(GTK_BOX(hbox), table, FALSE, FALSE, 0);
	gtk_widget_show(table);

	label = gtk_label_new("caotica2 BPM calculator");
	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 0, 1);
	gtk_widget_show(label);

	label = gtk_label_new("version 1.0");
	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 1, 2);
	gtk_widget_show(label);

	button = gtk_button_new_with_label("OK");
	gtk_table_attach_defaults(GTK_TABLE(table), button, 1, 2, 0, 1);
	g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(close_about), window);
	gtk_widget_show(button);

	label = gtk_label_new("copyright 2003");
	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 2, 3);
	gtk_widget_show(label);

	label = gtk_label_new("http://caotica2.free.fr/");
	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 3, 4);
	gtk_widget_show(label);

	button = gtk_button_new_with_label("Visit Site");
	gtk_table_attach_defaults(GTK_TABLE(table), button, 1, 2, 3, 4);
	g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(visit_site), NULL);
	gtk_widget_show(button);

	gdk_window_set_transient_for(window->window, _window->window);
	gtk_widget_show(window);
	return;
}

static void display_help(GtkButton *widget, gpointer data)
{
	system("dillo TOOBPM_1.0_english.html > /dev/null 2>&1 &");
	return;
}

static gboolean close_bpmcalc(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	gtk_main_quit();
	return(FALSE);
}

static void display_bpmcalc(void)
{
	GtkWidget	*table;
	GtkWidget	*hbox;
	GtkWidget	*label;
	GtkWidget	*button;
	GtkWidget	*pixmap;

	_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_resizable(GTK_WINDOW(_window), FALSE);
	gtk_window_set_title(GTK_WINDOW(_window), "caotica2 BPM Calculator 1.0");
	gtk_container_set_border_width(GTK_CONTAINER(_window), SPACING);
	g_signal_connect(G_OBJECT(_window), "delete_event", G_CALLBACK(close_bpmcalc), NULL);
	gtk_widget_realize(_window);

#if (GTK_MAJOR_VERSION == 1)
	_accel_group = gtk_accel_group_new();
	gtk_window_add_accel_group(GTK_WINDOW(_window), _accel_group);
#endif

	table = gtk_table_new(6, 3, FALSE);
	gtk_table_set_row_spacings(GTK_TABLE(table), SPACING);
	gtk_table_set_col_spacing(GTK_TABLE(table), 0, SPACING_2);
	gtk_table_set_col_spacing(GTK_TABLE(table), 1, SPACING);
	gtk_container_add(GTK_CONTAINER(_window), table);
	gtk_widget_show(table);

	label = gtk_label_new("Period (loop time) :");
	gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 0, 1);
	gtk_widget_show(label);

	hbox = gtk_hbox_new(FALSE, SPACING_2);
	gtk_table_attach_defaults(GTK_TABLE(table), hbox, 1, 2, 0, 1);
	gtk_widget_show(hbox);

	_period_entry = gtk_entry_new();
	gtk_entry_set_text(GTK_ENTRY(_period_entry), "2");
	gtk_box_pack_start(GTK_BOX(hbox), _period_entry, FALSE, FALSE, 0);
	gtk_widget_show(_period_entry);

	label = gtk_label_new("s");
	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
	gtk_widget_show(label);

	button = gtk_button_new_with_mnemonic("Compute _Tempo");
	gtk_table_attach_defaults(GTK_TABLE(table), button, 2, 3, 0, 1);
	g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(compute_tempo), NULL);
	gtk_widget_show(button);

	label = gtk_label_new("at :");
	gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 1, 2);
	gtk_widget_show(label);

	hbox = gtk_hbox_new(FALSE, SPACING_2);
	gtk_table_attach_defaults(GTK_TABLE(table), hbox, 1, 2, 1, 2);
	gtk_widget_show(hbox);

	_beats_entry = gtk_entry_new();
	gtk_entry_set_text(GTK_ENTRY(_beats_entry), "4");
	gtk_box_pack_start(GTK_BOX(hbox), _beats_entry, FALSE, FALSE, 0);
	gtk_widget_show(_beats_entry);

	label = gtk_label_new("beats");
	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
	gtk_widget_show(label);

	label = gtk_label_new("Tempo :");
	gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 2, 3);
	gtk_widget_show(label);

	hbox = gtk_hbox_new(FALSE, SPACING_2);
	gtk_table_attach_defaults(GTK_TABLE(table), hbox, 1, 2, 2, 3);
	gtk_widget_show(hbox);

	_tempo_entry = gtk_entry_new();
	gtk_entry_set_text(GTK_ENTRY(_tempo_entry), "120");
	gtk_box_pack_start(GTK_BOX(hbox), _tempo_entry, FALSE, FALSE, 0);
	gtk_widget_show(_tempo_entry);

	label = gtk_label_new("BPM");
	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
	gtk_widget_show(label);

	button = gtk_button_new_with_mnemonic("Compute _Period");
	gtk_table_attach_defaults(GTK_TABLE(table), button, 2, 3, 2, 3);
	g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(compute_period), NULL);
	gtk_widget_show(button);

	_nth_check = gtk_check_button_new_with_mnemonic("_Display n/th");
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_nth_check), FALSE);
	gtk_table_attach_defaults(GTK_TABLE(table), _nth_check, 1, 2, 3, 4);
	gtk_widget_show(_nth_check);

	_round_check = gtk_check_button_new_with_mnemonic("_Rounded results");
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_round_check), TRUE);
	gtk_table_attach_defaults(GTK_TABLE(table), _round_check, 1, 2, 4, 5);
	gtk_widget_show(_round_check);

	button = gtk_button_new_with_mnemonic("_About");
	gtk_table_attach(GTK_TABLE(table), button, 0, 1, 5, 6, GTK_FILL, GTK_SHRINK, 0, 0);
	g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(display_about), NULL);
	gtk_widget_show(button);

	pixmap = build_pixmap(_window, (gchar **) _caotica2_xpm, FALSE);
	gtk_table_attach_defaults(GTK_TABLE(table), pixmap, 1, 2, 5, 6);
	gtk_widget_show(pixmap);

	button = gtk_button_new_with_mnemonic("_Help");
	gtk_table_attach(GTK_TABLE(table), button, 2, 3, 5, 6, GTK_FILL, GTK_SHRINK, 0, 0);
	g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(display_help), NULL);
	gtk_widget_show(button);

	build_pixmap(_window, (gchar **) _bpmcalc_xpm, TRUE);
	gtk_widget_show(_window);
	return;
}

int main(int argc, char *argv[])
{
	gtk_init(&argc, &argv);
	display_bpmcalc();
	gtk_main();
	return(0);
}
